/*
    OpenChart2 Java Charting Library and Toolkit
    Copyright (C) 2005-2008 Approximatrix, LLC
    Copyright (C) 2001  Sebastian M�ller
    http://www.approximatrix.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    DotChart.java

 */

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionListener;

import com.approximatrix.charting.coordsystem.BoxCoordSystem;
import com.approximatrix.charting.coordsystem.CoordSystem;
import com.approximatrix.charting.model.threedimensional.ColorDotPlotDataModel;
import com.approximatrix.charting.model.threedimensional.ColorScaleDotPlotDataModel;
import com.approximatrix.charting.swing.ExtendedChartPanel;
import com.approximatrix.charting.render.ImageRenderer;
import com.approximatrix.charting.render.colorscale.SimpleColorScale;


/** Creates a dialog that contains a dot plot with 250,000 data points.  The
 * color is determined via a SimpleColorScale object and the x-axis position
 * values.  Zooming is enabled in this example; drag a box over a region to
 * zoom in, and right-click to zoom out.
 *  
 * @author armstrong
 *
 */
public class DotChart extends JDialog implements ActionListener {
	
	/** A menu item for closing the dialog */
	JMenuItem m_exit = null;
	
	/** The panel containing our chart */
	ExtendedChartPanel chartPanel = null;
	
	/** Initializes the dialog and its chart in a ChartPanel dialog */
	public DotChart() {
		super();
		
		// Initialize GUI components
		this.setTitle("Openchart2: A Dot Plot...");
		
		// Make the dialog a reasonable size
		this.setSize(500, 400);
		
		// Allow resize!  It works seamlessly in Opechart2
		this.setResizable(true);
		
		// The content pane
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BorderLayout());
		this.setContentPane(contentPane);
		
		// Set up a menu on this dialog, just to show that everything plays
		// well together.
		JMenuBar menu_bar = new JMenuBar();
		JMenu file_menu = new JMenu();
		file_menu.setText("File");
		m_exit = new JMenuItem();
		m_exit.setText("Exit");
		file_menu.add(m_exit);
		menu_bar.add(file_menu);
		this.setJMenuBar(menu_bar);
		
		// Make sure that on window close, we dispose of this dialog
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		
		// Set up this class as the action listener for our one menu item
		m_exit.addActionListener(this);
		
		contentPane.add(getChartPanel(),BorderLayout.CENTER);
	}
	
	/** Constructs (if necessary) and returns a valid chart panel */
	private ExtendedChartPanel getChartPanel() {
		
		// Create the panel if necessary
		if(chartPanel == null) {
			
			// Create a MultiScatterDataModel
			double[] xval = getRandomDim(250000);
			ColorDotPlotDataModel model = new ColorScaleDotPlotDataModel(xval,xval,xval,new SimpleColorScale(0,100)); 
			
			// The model stores the data as an image of a specific width.
			// Set the desired resolution here.  By default, it is 500x500.
			// This resolution does not define the actual rendered image, only
			// the data model's storage resolution.
			model.setResolution(400, 400);
			
			// If you know your plot is 500x500 for example, it would be optimal
			// to set the resolution accordingly.
			
			model.setAutoScale(false);
			model.setMinimumValueY(new Double(100.0));
			model.setMaximumValueY(new Double(0.0));
			
			model.setMinimumValueX(new Double(0.0));
			model.setMaximumValueX(new Double(100.0));
			
			
			// Create an associated coordinate system object
			CoordSystem coord = new BoxCoordSystem(model);
			
			// Generate the chart panel
			chartPanel = new ExtendedChartPanel(model,"Random Data");
			
			// Add the coordinate system to the chart
			chartPanel.setCoordSystem(coord);
			
			// Add the chart renderer
			ImageRenderer renderer = new ImageRenderer(coord,model);
			
			// Set the scaling system for drawing the chart
			renderer.setScaleHint(Image.SCALE_REPLICATE);
			
			chartPanel.setLegend(null);
			
			// Add a renderer to the chart
			chartPanel.addChartRenderer(renderer, 0);
			chartPanel.enableZoom(true);
			
		}
		return chartPanel;
	}
	
	/** Listener for our single menu item and our two buttons.  The proper action
	 * is taken based on which component caused the event to occur.
	 */
	public void actionPerformed(java.awt.event.ActionEvent e) {
		// If this was our exit button responding, dispose of the dialog
		if(e.getSource() == m_exit)
			dispose();
		
	}
			
	private double[] getRandomDim(int length) {
		double[] ret = new double[length];
		//ret[0] = 20.0;
		for(int i=0;i<length;i++)
			ret[i] = Math.random()*100.0;
		return ret;
	}
	
	private Color[] getRandomColor(int length) {
		Color[] ret = new Color[length];
		for(int i=0;i<length;i++)
			ret[i] = Color.BLUE;
		return ret;
	}
	
	/** Main routine to start the demonstration dialog
	 * 
	 * @param args unnecessary command line arguments
	 */
	public static void main(String[] args) {
		
		// Make this application appear native
		try {
			UIManager.setLookAndFeel(
					UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) { 
			System.out.println("Change of LNF failed...");
		}
		
		// Construct our bar chart dialog
		DotChart temp = new DotChart();
		
		// Make the dialog visible
		temp.setVisible(true);
	}
}

